import {
    VStack,
    Box,
    Text,
    HStack,
    ScrollView,
    Flex,
    Button,
    CloseIcon,
    IconButton,
    TextArea,
    useToast,
    KeyboardAvoidingView
} from "native-base";
import { Note } from "../../types/note";
import NoteView from "./NoteView";
import { MessageIcon } from "../general/Icons";
import React, { useCallback, useState } from "react";
import { updateNote, deleteNote } from "../../stores/actions/notesThunks";
import { ConfirmDialog } from "../general/Dialogs";
import { AppDispatch, RootState } from "../../stores/store";
import { useDispatch, useSelector } from "react-redux";
import { InfoToast } from "../toast/InfoToast";
import { UserRole } from "../../stores/reducers/userSlice";



const NotesListView = (props: { notes: Note[], navigation: any, handleCreateComment: any, requestPending: boolean, height: number }) => {
    const notes = props.notes;

    // const userRole = useSelector((state: RootState) => state.user.role)
    const userId = useSelector((state: RootState) => state.user.userId)
    const userRole = useSelector((state: RootState) => state.user.role)

    console.log(userRole)

    const [newComment, setNewComment] = useState<string>("");
    const [replyingTo, setReplyingTo] = useState<{ messageId: string; userName: string } | null>(null);

    const toast = useToast();

    const [renderedNotes, setRenderedNotes] = useState(6);

    const cancelRef = React.useRef(null);

    const dispatch = useDispatch<AppDispatch>();

    const [confirmDialog, setConfirmDialog] = React.useState<{
        headerText?: string;
        bodyText?: string;
        confirmText?: string;
        confirmColor?: string;
        cancelRef?: any;
        onClose?: () => void;
        onSubmit?: () => void;
    } | null>(null);

    const handleReply = useCallback((messageId: string, userName: string) => {
        setReplyingTo({ messageId, userName });
    }, [setReplyingTo]);

    const handleEdit = useCallback((note: Note) => {
        if (!props.requestPending) {
            dispatch(updateNote(note));
        }
    }, [dispatch, props.requestPending]);

    const handleDelete = useCallback((messageId: string) => {
        if (!props.requestPending) {
            dispatch(deleteNote({ noteId: messageId }));
        }
    }, [dispatch, props.requestPending]);



    const handleScrollEnd = (event: any) => {
        const offsetY = event.nativeEvent.contentOffset.y;
        const contentHeight = event.nativeEvent.contentSize.height;
        const screenHeight = event.nativeEvent.layoutMeasurement.height;

        if (offsetY + screenHeight >= contentHeight - 20) {
            // Load more notes when the user is near the end of the list
            const nextBatchSize = 5; // Number of notes to render in the next batch

            if ((renderedNotes + nextBatchSize) > notes.length && (renderedNotes + nextBatchSize) != notes.length) {
                toast.closeAll()
                toast.show({
                    render: ({
                        id
                    }) => {
                        return <InfoToast text={"All notes loaded..."} onClose={() => toast.close(id)} />;
                    },
                    duration: 3000
                });
                setRenderedNotes(notes.length);

            }
            else if ((renderedNotes + nextBatchSize) != notes.length) {
                toast.closeAll()
                toast.show({
                    render: ({
                        id
                    }) => {
                        return <InfoToast text={"Loading more notes..."} onClose={() => toast.close(id)} />;
                    },
                    duration: 3000
                });
                setRenderedNotes(renderedNotes + nextBatchSize);

            }
        }
    };

    return (
        <KeyboardAvoidingView
            h={ {lg: "auto"} }
            behavior={ "padding" }
            keyboardVerticalOffset={ 600 }
            mb={ 2 }
        >
        <VStack >
            <Box height={props.height - (replyingTo ? 40 : 0) ?? undefined} marginBottom={2}>
                {/* Notes */}
                <ScrollView contentContainerStyle={{ flexGrow: 1 }} onScrollEndDrag={handleScrollEnd} paddingLeft={2.5} paddingRight={2.5}>
                    {
                        notes && notes.length > 0 ? (
                            <VStack>
                                {
                                    notes.slice(0, renderedNotes).map((note, index) => {

                                        return <NoteView higlighted={replyingTo?.messageId == note.uuid} key={index} note={note}
                                            handleReply={
                                                handleReply
                                            }

                                            handleDelete={
                                                (note.username == userId || userRole == UserRole.Admin) ?
                                                    handleDelete
                                                    :
                                                    undefined
                                            }

                                            handleEdit={
                                                (note.username == userId || userRole == UserRole.Admin) ?
                                                    handleEdit
                                                    :
                                                    undefined
                                            } 
                                            setConfirmDialog={setConfirmDialog} navigation={props.navigation} />;

                                    })
                                }
                            </VStack>
                        ) : (
                            <Text>There are no notes.</Text>
                        )
                    }
                </ScrollView>
            </Box>

            {/* Create comment */}
            {
                props.handleCreateComment != null &&
                <VStack>

                    {/* Replying to */}
                    {replyingTo != null &&
                        <Flex direction="row" alignItems="center" justify="flex-end" height={10} backgroundColor={"secondary.50"}>
                            <Text>Replying to {replyingTo.userName}</Text>
                            <IconButton onPress={() => setReplyingTo(null)} size="sm" icon={<CloseIcon />} marginLeft={-1} marginRight={20} />
                        </Flex>
                    }


                    {/* Add comment */}
                    <HStack height={90} marginLeft={1.5} marginRight={1.5}>
                        <TextArea
                            flex={1}
                            placeholder={userRole == UserRole.Unauthorized || userRole == UserRole.User ? "Unathorized to add comment" : "Add comment"}
                            isDisabled={userRole == UserRole.Unauthorized || userRole == UserRole.User}
                            fontSize={14}
                            value={newComment}
                            onChangeText={setNewComment}
                            autoCompleteType={undefined}


                        />
                        <Flex marginLeft={2} marginBottom={8}>
                            <Button disabled={userRole == UserRole.Unauthorized || userRole == UserRole.User} onPress={() => props.handleCreateComment(newComment, replyingTo)} startIcon={<MessageIcon color="#FFF" />} />
                        </Flex>
                    </HStack>
                </VStack>
            }
            <ConfirmDialog {...confirmDialog} isShown={confirmDialog != null} cancelRef={cancelRef} />
        </VStack >
        </KeyboardAvoidingView>
    );
}

export default NotesListView